//const JSONEditor = require('jsoneditor');

const fs = require("fs");

var console = null;
var cfg = null;
var editor = null;

function onConfChange() {
    console.log('change:' + JSON.stringify((this)));
}

function onConfEditable() {
    console.log('edit:' + JSON.stringify((this)));
    return true;
}

function onError() {
    console.log('error: ' + JSON.stringify((this)));
}

function initEditor(div, _cfg, _console) {
    console = _console;
    cfg = _cfg;
    global.console = console;
    if (editor) {
        editor.destroy();
        editor = null;
    }
    div.innerHTML = "";
    const schema = {
        title: "ODC Schema",
        type: "object",
        properties: {
            path: {
                type: "string",
                is_dir_path: true,
            },
            cloud: {
                type: "object",
                properties: {
                    proto: {
                        enum: ["http", "https"],
                    },
                    hostname: {
                        type: "string",
                    },
                    port: {
                        type: "integer",
                    },
                    rel_path: {
                        type: "string",
                    },
                    key: {
                        type: "string",
                        is_path: true,
                    },
                    cert: {
                        type: "string",
                        is_path: true,
                    }
                },
                nb_data_per_frame: {
                    type: "integer"
                },
                frequency: {
                    type: "integer",
                    description: "Set upload frequency in seconds."
                },
            },
            spcu: {
                type: "object",
                properties: {
                    proto: {
                        enum: ["http", "https"],
                    },
                    port: {
                        type: "integer"
                    },
                    key: {
                        type: "string",
                        is_path: true,
                    },
                    cert: {
                        type: "string",
                        is_path: true,
                    },
                    ca: {
                        type: "string",
                        is_path: true,
                    }
                }
            }
        },
        required: ["nb_data_per_frame", "frequency", "path"]
    };
    div.innerHTML = ""
    editor = new JSONEditor(div, {
        search: false,
        mode: 'form',
        history: false,
        name: "Data Collector Configuration",
        onError: onError,
        onEditable: onConfEditable,
        onChange: onConfChange,
        schemaRefs: null,
        schema: schema
    }, _cfg);
    editor.expandAll();
}

function saveEditorConfig(fname) {
    cfg = editor.get();
    saveConfig(fname, editor.get());
    return cfg;
}

function saveConfig(fname, datas) {
    fs.writeFileSync(fname, JSON.stringify(datas, null, 4));
}

function updateDlCfg(fname, name, value, err) {
    switch (name) {
        case 'port':
            if (value != parseInt(value, 10)) {
                if (err) err.value = "This value must be and integer."
                return false;
            }
            cfg.spcu.port = parseInt(value, 10);
            break;
        case 'path':
            cfg.path = value; 
            break;
    }
    
    saveConfig(fname, cfg);
    return true;
}

function updateUlCfg(fname, name, value, err) {
    switch (name) {
        case 'nb_per_sframe':
            if (value != parseInt(value, 10) && value > 5000) {
                if (err) err.value = "This value must be between 0 and 5000."
                return false;
            }
            cfg.cloud.nb_data_per_frame = parseInt(value, 10);
            break;
        case 'period':
            if (value != parseInt(value, 10)) {
                if (err) err.value = "This value must be and integer."
                return false;
            }
            cfg.cloud.frequency = parseInt(value, 10);
            break;
    }
    saveConfig(fname, cfg);
    return true;
}

function initCfg(_cfg) {
    cfg = _cfg;
}

module.exports = {
    initCfg: initCfg,
    initEditor: initEditor,
    saveEditorConfig: saveEditorConfig,
    updateDlCfg: updateDlCfg,
    updateUlCfg: updateUlCfg,
};
